// Tide Protocol - Infrastructure for the Personal Data economy
// Copyright (C) 2019 Tide Foundation Ltd
// 
// This program is free software and is subject to the terms of 
// the Tide Community Open Source License as published by the 
// Tide Foundation Limited. You may modify it and redistribute 
// it in accordance with and subject to the terms of that License.
// This program is distributed WITHOUT WARRANTY of any kind, 
// including without any implied warranty of MERCHANTABILITY or 
// FITNESS FOR A PARTICULAR PURPOSE.
// See the Tide Community Open Source License for more details.
// You should have received a copy of the Tide Community Open 
// Source License along with this program.
// If not, see https://tide.org/licenses_tcosl-1-0-en

import BigInt from 'big-integer';

export default class ByteSerializer {
    /**
     * @param {Uint8Array} bytes
     * @returns {bigInt.BigInteger[]}
     */
    static DeserializeNumbers(bytes, countSize = 1) {
        return ByteSerializer.deserializeBytes(bytes, countSize).map(data => BigInt.fromArray(Array.from(data), 256, false))
    }

    /**
     * @param {Uint8Array} bytes
     * @returns {Uint8Array[]}
     */
    static deserializeBytes(bytes, countSize = 1) {
        if (bytes === null || bytes.length < 1 || (bytes[0] === 0 && bytes.length > 1))
            throw new Error("Format Error");

        var numItems = bytes[0];
        if (numItems === 0)
            return [];

        var lenIdx = 1;
        var numbers = []; 
        for (var i = 0; i < numItems; i++)
        {
            if (lenIdx >= bytes.length || lenIdx + countSize > bytes.length)
                throw new Error("Format Error");

            var nextBytes = ToInt(bytes.slice(lenIdx).slice(0, countSize));

            if (lenIdx + countSize + nextBytes > bytes.length)
                throw new Error("Format Error");

            numbers.push(bytes.slice(lenIdx + countSize).slice(0, nextBytes));
            lenIdx += nextBytes + countSize;
        }

        if (lenIdx != bytes.length)
            throw new Error("Format Error");

        return numbers;
    }
}

/**
 * @param {Uint8Array} bytes
 * @returns {number}
 */
function ToInt(bytes)
{
    return BigInt.fromArray(Array.from(bytes), 256, false).valueOf();
}
