// Tide Protocol - Infrastructure for the Personal Data economy
// Copyright (C) 2019 Tide Foundation Ltd
// 
// This program is free software and is subject to the terms of 
// the Tide Community Open Source License as published by the 
// Tide Foundation Limited. You may modify it and redistribute 
// it in accordance with and subject to the terms of that License.
// This program is distributed WITHOUT WARRANTY of any kind, 
// including without any implied warranty of MERCHANTABILITY or 
// FITNESS FOR A PARTICULAR PURPOSE.
// See the Tide Community Open Source License for more details.
// You should have received a copy of the Tide Community Open 
// Source License along with this program.
// If not, see https://tide.org/licenses_tcosl-1-0-en

import EcScalar from './ecScalar';
import EcPoint from './ecPoint';

const publicHeader = "MFYwEAYHKoZIzj0CAQYFK4EEAAoDQgAE";
const privateHeader = "MHQCAQEEIA==";

export default class EcDSAFormat {
    /**
     * @param {EcPoint} pub
     */
    static FormatPublicPem(pub) {
        return Buffer.concat([Buffer.from(publicHeader, 'base64'), Buffer.from(pub.toArray())]).toString("base64");
    }

    /**
     * @param {EcScalar} prv
     * @param {EcPoint} pub
     */
    static FormatPrivatePem(prv, pub = EcPoint.g.times(prv)) {
        return Buffer.concat([Buffer.from(privateHeader, 'base64')
            , Buffer.from(prv.toArray())
            , Buffer.from("oAcGBSuBBAAKoUQDQgAE", 'base64')
            , Buffer.from(pub.toArray())]).toString("base64");
    }
}
