// Tide Protocol - Infrastructure for the Personal Data economy
// Copyright (C) 2019 Tide Foundation Ltd
// 
// This program is free software and is subject to the terms of 
// the Tide Community Open Source License as published by the 
// Tide Foundation Limited. You may modify it and redistribute 
// it in accordance with and subject to the terms of that License.
// This program is distributed WITHOUT WARRANTY of any kind, 
// including without any implied warranty of MERCHANTABILITY or 
// FITNESS FOR A PARTICULAR PURPOSE.
// See the Tide Community Open Source License for more details.
// You should have received a copy of the Tide Community Open 
// Source License along with this program.
// If not, see https://tide.org/licenses_tcosl-1-0-en

import { n } from './eccParams';
import utils from './utils';
import BaseScalar from './baseScalar';
import BigInt from 'big-integer';

export default class EcScalar extends BaseScalar {
    /**
     * @param {(bigInt.BigInteger|number)} value
     */
    constructor(value) {
        super(value, n);
    }

    static get zero() { return new EcScalar(BigInt.zero) }
    
    static get one() { return new EcScalar(BigInt.one) }

    /**
     * @returns {number[]}
     */
    toArray() {
        return utils.padLeft(this.value.toArray(256).value, 32);
    }
}
